@@ -14,6 +14,11 @@ import stat
 import threading
 
 from .i18n import _
+from .pycompat import (
+    delattr,
+    getattr,
+    setattr,
+)
 from . import (
     encoding,
     error,
@@ -22,29 +27,32 @@ from . import (
     util,
 )
 
+
 def _avoidambig(path, oldstat):
     """Avoid file stat ambiguity forcibly
 
     This function causes copying ``path`` file, if it is owned by
     another (see issue5418 and issue5584 for detail).
     """
+
     def checkandavoid():
         newstat = util.filestat.frompath(path)
         # return whether file stat ambiguity is (already) avoided
-        return (not newstat.isambig(oldstat) or
-                newstat.avoidambig(path, oldstat))
+        return not newstat.isambig(oldstat) or newstat.avoidambig(path, oldstat)
+
     if not checkandavoid():
         # simply copy to change owner of path to get privilege to
         # advance mtime (see issue5418)
         util.rename(util.mktempcopy(path), path)
         checkandavoid()
 
+
 class abstractvfs(object):
     """Abstract base class; cannot be instantiated"""
 
     def __init__(self, *args, **kwargs):
         '''Prevent instantiation; don't call this from subclasses.'''
-        raise NotImplementedError('attempted instantiating ' + str(type(self)))
+        raise NotImplementedError(b'attempted instantiating ' + str(type(self)))
 
     def _auditpath(self, path, mode):
         raise NotImplementedError
@@ -56,9 +64,9 @@ class abstractvfs(object):
         except IOError as inst:
             if inst.errno != errno.ENOENT:
                 raise
-        return ""
+        return b""
 
-    def tryreadlines(self, path, mode='rb'):
+    def tryreadlines(self, path, mode=b'rb'):
         '''gracefully return an empty array for missing files'''
         try:
             return self.readlines(path, mode=mode)
@@ -78,23 +86,23 @@ class abstractvfs(object):
         return self.__call__
 
     def read(self, path):
-        with self(path, 'rb') as fp:
+        with self(path, b'rb') as fp:
             return fp.read()
 
-    def readlines(self, path, mode='rb'):
+    def readlines(self, path, mode=b'rb'):
         with self(path, mode=mode) as fp:
             return fp.readlines()
 
     def write(self, path, data, backgroundclose=False, **kwargs):
-        with self(path, 'wb', backgroundclose=backgroundclose, **kwargs) as fp:
+        with self(path, b'wb', backgroundclose=backgroundclose, **kwargs) as fp:
             return fp.write(data)
 
-    def writelines(self, path, data, mode='wb', notindexed=False):
+    def writelines(self, path, data, mode=b'wb', notindexed=False):
         with self(path, mode=mode, notindexed=notindexed) as fp:
             return fp.writelines(data)
 
     def append(self, path, data):
-        with self(path, 'ab') as fp:
+        with self(path, b'ab') as fp:
             return fp.write(data)
 
     def basename(self, path):
@@ -172,9 +180,10 @@ class abstractvfs(object):
     def mkdir(self, path=None):
         return os.mkdir(self.join(path))
 
-    def mkstemp(self, suffix='', prefix='tmp', dir=None):
-        fd, name = pycompat.mkstemp(suffix=suffix, prefix=prefix,
-                                    dir=self.join(dir))
+    def mkstemp(self, suffix=b'', prefix=b'tmp', dir=None):
+        fd, name = pycompat.mkstemp(
+            suffix=suffix, prefix=prefix, dir=self.join(dir)
+        )
         dname, fname = util.split(name)
         if dir:
             return fd, os.path.join(dir, fname)
@@ -199,7 +208,7 @@ class abstractvfs(object):
         checkambig=True only in limited cases (see also issue5418 and
         issue5584 for detail).
         """
-        self._auditpath(dst, 'w')
+        self._auditpath(dst, b'w')
         srcpath = self.join(src)
         dstpath = self.join(dst)
         oldstat = checkambig and util.filestat.frompath(dstpath)
@@ -227,6 +236,7 @@ class abstractvfs(object):
         If ``forcibly``, this tries to remove READ-ONLY files, too.
         """
         if forcibly:
+
             def onerror(function, path, excinfo):
                 if function is not os.remove:
                     raise
@@ -236,10 +246,12 @@ class abstractvfs(object):
                     raise
                 os.chmod(path, stat.S_IMODE(s.st_mode) | stat.S_IWRITE)
                 os.remove(path)
+
         else:
             onerror = None
-        return shutil.rmtree(self.join(path),
-                             ignore_errors=ignore_errors, onerror=onerror)
+        return shutil.rmtree(
+            self.join(path), ignore_errors=ignore_errors, onerror=onerror
+        )
 
     def setflags(self, path, l, x):
         return util.setflags(self.join(path), l, x)
@@ -255,8 +267,9 @@ class abstractvfs(object):
         util.tryunlink(self.join(path))
 
     def unlinkpath(self, path=None, ignoremissing=False, rmdir=True):
-        return util.unlinkpath(self.join(path), ignoremissing=ignoremissing,
-                               rmdir=rmdir)
+        return util.unlinkpath(
+            self.join(path), ignoremissing=ignoremissing, rmdir=rmdir
+        )
 
     def utime(self, path=None, t=None):
         return os.utime(self.join(path), t)
@@ -294,7 +307,8 @@ class abstractvfs(object):
         vfs = getattr(self, 'vfs', self)
         if getattr(vfs, '_backgroundfilecloser', None):
             raise error.Abort(
-                _('can only have 1 active background file closer'))
+                _(b'can only have 1 active background file closer')
+            )
 
         with backgroundfilecloser(ui, expectedcount=expectedcount) as bfc:
             try:
@@ -303,6 +317,7 @@ class abstractvfs(object):
             finally:
                 vfs._backgroundfilecloser = None
 
+
 class vfs(abstractvfs):
     '''Operate files relative to a base directory
 
@@ -313,8 +328,15 @@ class vfs(abstractvfs):
     (b) the base directory is managed by hg and considered sort-of append-only.
     See pathutil.pathauditor() for details.
     '''
-    def __init__(self, base, audit=True, cacheaudited=False, expandpath=False,
-                 realpath=False):
+
+    def __init__(
+        self,
+        base,
+        audit=True,
+        cacheaudited=False,
+        expandpath=False,
+        realpath=False,
+    ):
         if expandpath:
             base = util.expandpath(base)
         if realpath:
@@ -324,9 +346,10 @@ class vfs(abstractvfs):
         if audit:
             self.audit = pathutil.pathauditor(self.base, cached=cacheaudited)
         else:
-            self.audit = (lambda path, mode=None: True)
+            self.audit = lambda path, mode=None: True
         self.createmode = None
         self._trustnlink = None
+        self.options = {}
 
     @util.propertycache
     def _cansymlink(self):
@@ -347,12 +370,20 @@ class vfs(abstractvfs):
                 path = os.path.relpath(path, self.base)
             r = util.checkosfilename(path)
             if r:
-                raise error.Abort("%s: %r" % (r, path))
+                raise error.Abort(b"%s: %r" % (r, path))
             self.audit(path, mode=mode)
 
-    def __call__(self, path, mode="r", atomictemp=False, notindexed=False,
-                 backgroundclose=False, checkambig=False, auditpath=True,
-                 makeparentdirs=True):
+    def __call__(
+        self,
+        path,
+        mode=b"r",
+        atomictemp=False,
+        notindexed=False,
+        backgroundclose=False,
+        checkambig=False,
+        auditpath=True,
+        makeparentdirs=True,
+    ):
         '''Open ``path`` file, which is relative to vfs root.
 
         By default, parent directories are created as needed. Newly created
@@ -387,11 +418,11 @@ class vfs(abstractvfs):
             self._auditpath(path, mode)
         f = self.join(path)
 
-        if "b" not in mode:
-            mode += "b" # for that other OS
+        if b"b" not in mode:
+            mode += b"b"  # for that other OS
 
         nlink = -1
-        if mode not in ('r', 'rb'):
+        if mode not in (b'r', b'rb'):
             dirname, basename = util.split(f)
             # If basename is empty, then the path is malformed because it points
             # to a directory. Let the posixfile() call below raise IOError.
@@ -399,10 +430,11 @@ class vfs(abstractvfs):
                 if atomictemp:
                     if makeparentdirs:
                         util.makedirs(dirname, self.createmode, notindexed)
-                    return util.atomictempfile(f, mode, self.createmode,
-                                               checkambig=checkambig)
+                    return util.atomictempfile(
+                        f, mode, self.createmode, checkambig=checkambig
+                    )
                 try:
-                    if 'w' in mode:
+                    if b'w' in mode:
                         util.unlink(f)
                         nlink = 0
                     else:
@@ -411,7 +443,7 @@ class vfs(abstractvfs):
                         with util.posixfile(f):
                             nlink = util.nlinks(f)
                             if nlink < 1:
-                                nlink = 2 # force mktempcopy (issue1922)
+                                nlink = 2  # force mktempcopy (issue1922)
                 except (OSError, IOError) as e:
                     if e.errno != errno.ENOENT:
                         raise
@@ -428,17 +460,26 @@ class vfs(abstractvfs):
             self._fixfilemode(f)
 
         if checkambig:
-            if mode in ('r', 'rb'):
-                raise error.Abort(_('implementation error: mode %s is not'
-                                    ' valid for checkambig=True') % mode)
+            if mode in (b'r', b'rb'):
+                raise error.Abort(
+                    _(
+                        b'implementation error: mode %s is not'
+                        b' valid for checkambig=True'
+                    )
+                    % mode
+                )
             fp = checkambigatclosing(fp)
 
-        if (backgroundclose and
-                isinstance(threading.currentThread(), threading._MainThread)):
+        if backgroundclose and isinstance(
+            threading.currentThread(), threading._MainThread
+        ):
             if not self._backgroundfilecloser:
-                raise error.Abort(_('backgroundclose can only be used when a '
-                                  'backgroundclosing context manager is active')
-                                  )
+                raise error.Abort(
+                    _(
+                        b'backgroundclose can only be used when a '
+                        b'backgroundclosing context manager is active'
+                    )
+                )
 
             fp = delayclosedfile(fp, self._backgroundfilecloser)
 
@@ -455,9 +496,12 @@ class vfs(abstractvfs):
             try:
                 os.symlink(src, linkname)
             except OSError as err:
-                raise OSError(err.errno, _('could not symlink to %r: %s') %
-                              (src, encoding.strtolocal(err.strerror)),
-                              linkname)
+                raise OSError(
+                    err.errno,
+                    _(b'could not symlink to %r: %s')
+                    % (src, encoding.strtolocal(err.strerror)),
+                    linkname,
+                )
         else:
             self.write(dst, src)
 
@@ -467,8 +511,10 @@ class vfs(abstractvfs):
         else:
             return self.base
 
+
 opener = vfs
 
+
 class proxyvfs(abstractvfs):
     def __init__(self, vfs):
         self.vfs = vfs
@@ -484,6 +530,7 @@ class proxyvfs(abstractvfs):
     def options(self, value):
         self.vfs.options = value
 
+
 class filtervfs(proxyvfs, abstractvfs):
     '''Wrapper vfs for filtering filenames with a function.'''
 
@@ -500,27 +547,31 @@ class filtervfs(proxyvfs, abstractvfs):
         else:
             return self.vfs.join(path)
 
+
 filteropener = filtervfs
 
+
 class readonlyvfs(proxyvfs):
     '''Wrapper vfs preventing any writing.'''
 
     def __init__(self, vfs):
         proxyvfs.__init__(self, vfs)
 
-    def __call__(self, path, mode='r', *args, **kw):
-        if mode not in ('r', 'rb'):
-            raise error.Abort(_('this vfs is read only'))
+    def __call__(self, path, mode=b'r', *args, **kw):
+        if mode not in (b'r', b'rb'):
+            raise error.Abort(_(b'this vfs is read only'))
         return self.vfs(path, mode, *args, **kw)
 
     def join(self, path, *insidef):
         return self.vfs.join(path, *insidef)
 
+
 class closewrapbase(object):
     """Base class of wrapper, which hooks closing
 
     Do not instantiate outside of the vfs layer.
     """
+
     def __init__(self, fh):
         object.__setattr__(self, r'_origfh', fh)
 
@@ -538,16 +589,18 @@ class closewrapbase(object):
         return self
 
     def __exit__(self, exc_type, exc_value, exc_tb):
-        raise NotImplementedError('attempted instantiating ' + str(type(self)))
+        raise NotImplementedError(b'attempted instantiating ' + str(type(self)))
 
     def close(self):
-        raise NotImplementedError('attempted instantiating ' + str(type(self)))
+        raise NotImplementedError(b'attempted instantiating ' + str(type(self)))
+
 
 class delayclosedfile(closewrapbase):
     """Proxy for a file object whose close is delayed.
 
     Do not instantiate outside of the vfs layer.
     """
+
     def __init__(self, fh, closer):
         super(delayclosedfile, self).__init__(fh)
         object.__setattr__(self, r'_closer', closer)
@@ -558,8 +611,10 @@ class delayclosedfile(closewrapbase):
     def close(self):
         self._closer.close(self._origfh)
 
+
 class backgroundfilecloser(object):
     """Coordinates background closing of file handles on multiple threads."""
+
     def __init__(self, ui, expectedcount=-1):
         self._running = False
         self._entered = False
@@ -569,7 +624,7 @@ class backgroundfilecloser(object):
         # Only Windows/NTFS has slow file closing. So only enable by default
         # on that platform. But allow to be enabled elsewhere for testing.
         defaultenabled = pycompat.iswindows
-        enabled = ui.configbool('worker', 'backgroundclose', defaultenabled)
+        enabled = ui.configbool(b'worker', b'backgroundclose', defaultenabled)
 
         if not enabled:
             return
@@ -577,23 +632,24 @@ class backgroundfilecloser(object):
         # There is overhead to starting and stopping the background threads.
         # Don't do background processing unless the file count is large enough
         # to justify it.
-        minfilecount = ui.configint('worker', 'backgroundcloseminfilecount')
+        minfilecount = ui.configint(b'worker', b'backgroundcloseminfilecount')
         # FUTURE dynamically start background threads after minfilecount closes.
         # (We don't currently have any callers that don't know their file count)
         if expectedcount > 0 and expectedcount < minfilecount:
             return
 
-        maxqueue = ui.configint('worker', 'backgroundclosemaxqueue')
-        threadcount = ui.configint('worker', 'backgroundclosethreadcount')
+        maxqueue = ui.configint(b'worker', b'backgroundclosemaxqueue')
+        threadcount = ui.configint(b'worker', b'backgroundclosethreadcount')
 
-        ui.debug('starting %d threads for background file closing\n' %
-                 threadcount)
+        ui.debug(
+            b'starting %d threads for background file closing\n' % threadcount
+        )
 
         self._queue = pycompat.queue.Queue(maxsize=maxqueue)
         self._running = True
 
         for i in range(threadcount):
-            t = threading.Thread(target=self._worker, name='backgroundcloser')
+            t = threading.Thread(target=self._worker, name=b'backgroundcloser')
             self._threads.append(t)
             t.start()
 
@@ -628,8 +684,9 @@ class backgroundfilecloser(object):
     def close(self, fh):
         """Schedule a file for closing."""
         if not self._entered:
-            raise error.Abort(_('can only call close() when context manager '
-                              'active'))
+            raise error.Abort(
+                _(b'can only call close() when context manager active')
+            )
 
         # If a background thread encountered an exception, raise now so we fail
         # fast. Otherwise we may potentially go on for minutes until the error
@@ -646,6 +703,7 @@ class backgroundfilecloser(object):
 
         self._queue.put(fh, block=True, timeout=None)
 
+
 class checkambigatclosing(closewrapbase):
     """Proxy for a file object, to avoid ambiguity of file stat
 
@@ -656,6 +714,7 @@ class checkambigatclosing(closewrapbase):
 
     Do not instantiate outside of the vfs layer.
     """
+
     def __init__(self, fh):
         super(checkambigatclosing, self).__init__(fh)
         object.__setattr__(self, r'_oldstat', util.filestat.frompath(fh.name))
